/****************************************************************************
** Meta object code from reading C++ file 'secretserviceclient.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../keepsecret-1.1.0/src/secretserviceclient.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'secretserviceclient.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19SecretServiceClientE_t {};
} // unnamed namespace

template <> constexpr inline auto SecretServiceClient::qt_create_metaobjectdata<qt_meta_tag_ZN19SecretServiceClientE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SecretServiceClient",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Cannot create elements of type SecretServiceClient",
        "serviceChanged",
        "",
        "promptClosed",
        "accepted",
        "collectionListDirty",
        "defaultCollectionChanged",
        "collection",
        "collectionCreated",
        "QDBusObjectPath",
        "path",
        "collectionDeleted",
        "collectionLocked",
        "collectionUnlocked",
        "handlePrompt",
        "dismissed",
        "onCollectionCreated",
        "onCollectionDeleted",
        "onPropertiesChanged",
        "interface",
        "QVariantMap",
        "changedProperties",
        "invalidatedProperties",
        "lockCollection",
        "collectionPath",
        "unlockCollection",
        "createCollection",
        "collectionName",
        "deleteCollection",
        "defaultCollection",
        "Type",
        "PlainText",
        "Base64",
        "Binary",
        "Map",
        "Unknown"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'serviceChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'promptClosed'
        QtMocHelpers::SignalData<void(bool)>(9, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Signal 'collectionListDirty'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'defaultCollectionChanged'
        QtMocHelpers::SignalData<void(const QString &)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'collectionCreated'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &)>(14, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'collectionDeleted'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &)>(17, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'collectionLocked'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &)>(18, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'collectionUnlocked'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &)>(19, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'handlePrompt'
        QtMocHelpers::SlotData<void(bool)>(20, 8, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 21 },
        }}),
        // Slot 'onCollectionCreated'
        QtMocHelpers::SlotData<void(const QDBusObjectPath &)>(22, 8, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'onCollectionDeleted'
        QtMocHelpers::SlotData<void(const QDBusObjectPath &)>(23, 8, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'onPropertiesChanged'
        QtMocHelpers::SlotData<void(const QString &, const QVariantMap &, const QStringList &)>(24, 8, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 25 }, { 0x80000000 | 26, 27 }, { QMetaType::QStringList, 28 },
        }}),
        // Method 'lockCollection'
        QtMocHelpers::MethodData<void(const QString &)>(29, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 30 },
        }}),
        // Method 'unlockCollection'
        QtMocHelpers::MethodData<void(const QString &)>(31, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 30 },
        }}),
        // Method 'createCollection'
        QtMocHelpers::MethodData<void(const QString &)>(32, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Method 'deleteCollection'
        QtMocHelpers::MethodData<void(const QString &)>(34, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 30 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'defaultCollection'
        QtMocHelpers::PropertyData<QString>(35, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Type'
        QtMocHelpers::EnumData<enum Type>(36, 36, QMC::EnumFlags{}).add({
            {   37, Type::PlainText },
            {   38, Type::Base64 },
            {   39, Type::Binary },
            {   40, Type::Map },
            {   41, Type::Unknown },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<SecretServiceClient, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject SecretServiceClient::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19SecretServiceClientE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19SecretServiceClientE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19SecretServiceClientE_t>.metaTypes,
    nullptr
} };

void SecretServiceClient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SecretServiceClient *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->serviceChanged(); break;
        case 1: _t->promptClosed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->collectionListDirty(); break;
        case 3: _t->defaultCollectionChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->collectionCreated((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 5: _t->collectionDeleted((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 6: _t->collectionLocked((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 7: _t->collectionUnlocked((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 8: _t->handlePrompt((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->onCollectionCreated((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 10: _t->onCollectionDeleted((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 11: _t->onPropertiesChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[3]))); break;
        case 12: _t->lockCollection((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->unlockCollection((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->createCollection((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->deleteCollection((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SecretServiceClient::*)()>(_a, &SecretServiceClient::serviceChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SecretServiceClient::*)(bool )>(_a, &SecretServiceClient::promptClosed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SecretServiceClient::*)()>(_a, &SecretServiceClient::collectionListDirty, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SecretServiceClient::*)(const QString & )>(_a, &SecretServiceClient::defaultCollectionChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SecretServiceClient::*)(const QDBusObjectPath & )>(_a, &SecretServiceClient::collectionCreated, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SecretServiceClient::*)(const QDBusObjectPath & )>(_a, &SecretServiceClient::collectionDeleted, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (SecretServiceClient::*)(const QDBusObjectPath & )>(_a, &SecretServiceClient::collectionLocked, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (SecretServiceClient::*)(const QDBusObjectPath & )>(_a, &SecretServiceClient::collectionUnlocked, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->defaultCollection(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDefaultCollection(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *SecretServiceClient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SecretServiceClient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19SecretServiceClientE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int SecretServiceClient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void SecretServiceClient::serviceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void SecretServiceClient::promptClosed(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void SecretServiceClient::collectionListDirty()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void SecretServiceClient::defaultCollectionChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void SecretServiceClient::collectionCreated(const QDBusObjectPath & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void SecretServiceClient::collectionDeleted(const QDBusObjectPath & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void SecretServiceClient::collectionLocked(const QDBusObjectPath & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void SecretServiceClient::collectionUnlocked(const QDBusObjectPath & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
