// This file is generated by kconfig_compiler_kf6 from keepsecretconfig.kcfg.
// All changes you do to this file will be lost.
#ifndef KEEPSECRETCONFIG_H
#define KEEPSECRETCONFIG_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>
#include <qqmlintegration.h>

class QQmlEngine;
class QJSEngine;

class KeepsecretConfig : public KConfigSkeleton
{
  Q_OBJECT
  QML_ELEMENT
  QML_SINGLETON
  public:

    static KeepsecretConfig *self();
    static KeepsecretConfig *create(QQmlEngine *, QJSEngine *);
    ~KeepsecretConfig() override;

    /**
      Set Some setting description
    */
    static
    void setSomeSetting( bool v )
    {
      if (v != self()->mSomeSetting && !self()->isSomeSettingImmutable()) {
        self()->mSomeSetting = v;
        Q_EMIT self()->someSettingChanged();
      }
    }

    Q_PROPERTY(bool someSetting READ someSetting WRITE setSomeSetting NOTIFY someSettingChanged)
    Q_PROPERTY(bool isSomeSettingImmutable READ isSomeSettingImmutable CONSTANT)
    Q_PROPERTY(bool defaultSomeSettingValue READ defaultSomeSettingValue CONSTANT)
    /**
      Get Some setting description
    */
    static
    bool someSetting()
    {
      return self()->mSomeSetting;
    }

    /**
      Is Some setting description Immutable
    */
    static
    bool isSomeSettingImmutable()
    {
      return self()->isImmutable( QStringLiteral( "someSetting" ) );
    }

    /**
      Get Some setting description default value
    */
    static
    bool defaultSomeSettingValue()
    {
        return defaultSomeSettingValue_helper();
    }


    enum {
      signalSomeSettingChanged = 1
    };

  Q_SIGNALS:
    void someSettingChanged();

  private:
    void itemChanged(quint64 signalFlag);

  protected:
    KeepsecretConfig(QObject *parent = nullptr);
    friend class KeepsecretConfigHelper;


    // General
    bool mSomeSetting;
    static bool defaultSomeSettingValue_helper();
};

#endif

