/****************************************************************************
** Meta object code from reading C++ file 'secretitemproxy.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../keepsecret-1.1.0/src/secretitemproxy.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'secretitemproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15SecretItemProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto SecretItemProxy::qt_create_metaobjectdata<qt_meta_tag_ZN15SecretItemProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SecretItemProxy",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Cannot create elements of type SecretItemProxy",
        "itemLoaded",
        "",
        "itemSaved",
        "creationTimeChanged",
        "time",
        "modificationTimeChanged",
        "walletChanged",
        "wallet",
        "folderChanged",
        "folder",
        "itemNameChanged",
        "itemName",
        "labelChanged",
        "secretValueChanged",
        "typeChanged",
        "SecretServiceClient::Type",
        "type",
        "attributesChanged",
        "QVariantMap",
        "attribures",
        "setAttribute",
        "key",
        "value",
        "copySecret",
        "createItem",
        "label",
        "secret",
        "user",
        "server",
        "collectionPath",
        "loadItem",
        "itemPath",
        "unlock",
        "save",
        "revert",
        "close",
        "deleteItem",
        "creationTime",
        "modificationTime",
        "secretValue",
        "formattedBinarySecret",
        "attributes"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'itemLoaded'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'itemSaved'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'creationTimeChanged'
        QtMocHelpers::SignalData<void(const QDateTime &)>(10, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDateTime, 11 },
        }}),
        // Signal 'modificationTimeChanged'
        QtMocHelpers::SignalData<void(const QDateTime &)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDateTime, 11 },
        }}),
        // Signal 'walletChanged'
        QtMocHelpers::SignalData<void(const QString &)>(13, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Signal 'folderChanged'
        QtMocHelpers::SignalData<void(const QString &)>(15, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Signal 'itemNameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(17, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Signal 'labelChanged'
        QtMocHelpers::SignalData<void(const QString &)>(19, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Signal 'secretValueChanged'
        QtMocHelpers::SignalData<void()>(20, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'typeChanged'
        QtMocHelpers::SignalData<void(SecretServiceClient::Type)>(21, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Signal 'attributesChanged'
        QtMocHelpers::SignalData<void(const QVariantMap &)>(24, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Method 'setAttribute'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(27, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 28 }, { QMetaType::QString, 29 },
        }}),
        // Method 'copySecret'
        QtMocHelpers::MethodData<void()>(30, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'createItem'
        QtMocHelpers::MethodData<void(const QString &, const QByteArray &, const QString &, const QString &, const QString &)>(31, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 32 }, { QMetaType::QByteArray, 33 }, { QMetaType::QString, 34 }, { QMetaType::QString, 35 },
            { QMetaType::QString, 36 },
        }}),
        // Method 'loadItem'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(37, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 36 }, { QMetaType::QString, 38 },
        }}),
        // Method 'unlock'
        QtMocHelpers::MethodData<void()>(39, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'save'
        QtMocHelpers::MethodData<void()>(40, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'revert'
        QtMocHelpers::MethodData<void()>(41, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'close'
        QtMocHelpers::MethodData<void()>(42, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'deleteItem'
        QtMocHelpers::MethodData<void()>(43, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'creationTime'
        QtMocHelpers::PropertyData<QDateTime>(44, QMetaType::QDateTime, QMC::DefaultPropertyFlags, 2),
        // property 'modificationTime'
        QtMocHelpers::PropertyData<QDateTime>(45, QMetaType::QDateTime, QMC::DefaultPropertyFlags, 3),
        // property 'wallet'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags, 4),
        // property 'itemName'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags, 6),
        // property 'label'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'secretValue'
        QtMocHelpers::PropertyData<QString>(46, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'type'
        QtMocHelpers::PropertyData<SecretServiceClient::Type>(23, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 9),
        // property 'formattedBinarySecret'
        QtMocHelpers::PropertyData<QString>(47, QMetaType::QString, QMC::DefaultPropertyFlags, 8),
        // property 'attributes'
        QtMocHelpers::PropertyData<QVariantMap>(48, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 10),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<SecretItemProxy, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN15SecretItemProxyE[] = {
    QMetaObject::SuperData::link<SecretServiceClient::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject SecretItemProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SecretItemProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SecretItemProxyE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN15SecretItemProxyE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15SecretItemProxyE_t>.metaTypes,
    nullptr
} };

void SecretItemProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SecretItemProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->itemLoaded(); break;
        case 1: _t->itemSaved(); break;
        case 2: _t->creationTimeChanged((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1]))); break;
        case 3: _t->modificationTimeChanged((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1]))); break;
        case 4: _t->walletChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->folderChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->itemNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->labelChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->secretValueChanged(); break;
        case 9: _t->typeChanged((*reinterpret_cast<std::add_pointer_t<SecretServiceClient::Type>>(_a[1]))); break;
        case 10: _t->attributesChanged((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 11: _t->setAttribute((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 12: _t->copySecret(); break;
        case 13: _t->createItem((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5]))); break;
        case 14: _t->loadItem((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 15: _t->unlock(); break;
        case 16: _t->save(); break;
        case 17: _t->revert(); break;
        case 18: _t->close(); break;
        case 19: _t->deleteItem(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SecretItemProxy::*)()>(_a, &SecretItemProxy::itemLoaded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SecretItemProxy::*)()>(_a, &SecretItemProxy::itemSaved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SecretItemProxy::*)(const QDateTime & )>(_a, &SecretItemProxy::creationTimeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SecretItemProxy::*)(const QDateTime & )>(_a, &SecretItemProxy::modificationTimeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SecretItemProxy::*)(const QString & )>(_a, &SecretItemProxy::walletChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SecretItemProxy::*)(const QString & )>(_a, &SecretItemProxy::folderChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (SecretItemProxy::*)(const QString & )>(_a, &SecretItemProxy::itemNameChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (SecretItemProxy::*)(const QString & )>(_a, &SecretItemProxy::labelChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (SecretItemProxy::*)()>(_a, &SecretItemProxy::secretValueChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (SecretItemProxy::*)(SecretServiceClient::Type )>(_a, &SecretItemProxy::typeChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (SecretItemProxy::*)(const QVariantMap & )>(_a, &SecretItemProxy::attributesChanged, 10))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDateTime*>(_v) = _t->creationTime(); break;
        case 1: *reinterpret_cast<QDateTime*>(_v) = _t->modificationTime(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->wallet(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->itemName(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->label(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->secretValue(); break;
        case 6: *reinterpret_cast<SecretServiceClient::Type*>(_v) = _t->type(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->formattedBinarySecret(); break;
        case 8: *reinterpret_cast<QVariantMap*>(_v) = _t->attributes(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 4: _t->setLabel(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setSecretValue(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *SecretItemProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SecretItemProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SecretItemProxyE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int SecretItemProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 20;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void SecretItemProxy::itemLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void SecretItemProxy::itemSaved()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void SecretItemProxy::creationTimeChanged(const QDateTime & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void SecretItemProxy::modificationTimeChanged(const QDateTime & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void SecretItemProxy::walletChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void SecretItemProxy::folderChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void SecretItemProxy::itemNameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void SecretItemProxy::labelChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void SecretItemProxy::secretValueChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void SecretItemProxy::typeChanged(SecretServiceClient::Type _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void SecretItemProxy::attributesChanged(const QVariantMap & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
