/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<app.h>)
#  include <app.h>
#endif
#if __has_include(<collectionmodel.h>)
#  include <collectionmodel.h>
#endif
#if __has_include(<keepsecretconfig.h>)
#  include <keepsecretconfig.h>
#endif
#if __has_include(<secretitemproxy.h>)
#  include <secretitemproxy.h>
#endif
#if __has_include(<secretserviceclient.h>)
#  include <secretserviceclient.h>
#endif
#if __has_include(<statetracker.h>)
#  include <statetracker.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_org_kde_keepsecret()
{
    qmlRegisterModule("org.kde.keepsecret", 254, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<App>("org.kde.keepsecret", 254);
    qmlRegisterTypesAndRevisions<CollectionModel>("org.kde.keepsecret", 254);
    qmlRegisterEnum<CollectionModel::Roles>("CollectionModel::Roles");
    qmlRegisterTypesAndRevisions<KeepsecretConfig>("org.kde.keepsecret", 254);
    QMetaType::fromType<QAbstractItemModel *>().id();
    qmlRegisterEnum<QAbstractItemModel::LayoutChangeHint>("QAbstractItemModel::LayoutChangeHint");
    qmlRegisterEnum<QAbstractItemModel::CheckIndexOption>("QAbstractItemModel::CheckIndexOption");
    QMetaType::fromType<QAbstractListModel *>().id();
    qmlRegisterTypesAndRevisions<SecretItemProxy>("org.kde.keepsecret", 254);
    qmlRegisterTypesAndRevisions<SecretServiceClient>("org.kde.keepsecret", 254);
    qmlRegisterEnum<SecretServiceClient::Type>("SecretServiceClient::Type");
    qmlRegisterTypesAndRevisions<StateTracker>("org.kde.keepsecret", 254);
    qmlRegisterEnum<StateTracker::State>("StateTracker::State");
    qmlRegisterEnum<StateTracker::Operation>("StateTracker::Operation");
    qmlRegisterEnum<StateTracker::Error>("StateTracker::Error");
    QT_WARNING_POP
    qmlRegisterModule("org.kde.keepsecret", 254, 254);
}

static const QQmlModuleRegistration orgkdekeepsecretRegistration("org.kde.keepsecret", qml_register_types_org_kde_keepsecret);
