/****************************************************************************
** Meta object code from reading C++ file 'statetracker.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../keepsecret-1.1.0/src/statetracker.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'statetracker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12StateTrackerE_t {};
} // unnamed namespace

template <> constexpr inline auto StateTracker::qt_create_metaobjectdata<qt_meta_tag_ZN12StateTrackerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StateTracker",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Cannot create elements of type StateTracker",
        "statusChanged",
        "",
        "StateTracker::Status",
        "oldStatus",
        "newStatus",
        "serviceConnectedChanged",
        "connected",
        "operationsChanged",
        "StateTracker::Operations",
        "oldOperations",
        "newOperations",
        "operationsReadableNameChanged",
        "name",
        "errorChanged",
        "Error",
        "error",
        "errorMessage",
        "status",
        "Status",
        "operations",
        "Operations",
        "operationsReadableName",
        "State",
        "ServiceDisconnected",
        "ServiceConnected",
        "ItemReady",
        "ItemLocked",
        "ItemNeedsSave",
        "CollectionReady",
        "CollectionLocked",
        "Operation",
        "OperationNone",
        "ServiceConnecting",
        "ServiceLoadingCollections",
        "ItemCreating",
        "ItemLoading",
        "ItemLoadingSecret",
        "ItemUnlocking",
        "ItemSaving",
        "ItemSavingLabel",
        "ItemSavingSecret",
        "ItemSavingAttributes",
        "ItemDeleting",
        "CollectionReadingDefault",
        "CollectionWritingDefault",
        "CollectionCreating",
        "CollectionLoading",
        "CollectionUnlocking",
        "CollectionLocking",
        "CollectionDeleting",
        "NoError",
        "ServiceConnectionError",
        "ServiceLoadCollectionsError",
        "ItemCreationError",
        "ItemLoadError",
        "ItemLoadSecretError",
        "ItemUnlockError",
        "ItemSaveError",
        "ItemDeleteError",
        "CollectionReadDefaultError",
        "CollectionWriteDefaultError",
        "CollectionCreationError",
        "CollectionLoadError",
        "CollectionUnlockError",
        "CollectionLockError",
        "CollectionDeleteError"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void(StateTracker::Status, StateTracker::Status)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 9, 11 },
        }}),
        // Signal 'serviceConnectedChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Signal 'operationsChanged'
        QtMocHelpers::SignalData<void(StateTracker::Operations, StateTracker::Operations)>(14, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { 0x80000000 | 15, 17 },
        }}),
        // Signal 'operationsReadableNameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(18, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Signal 'errorChanged'
        QtMocHelpers::SignalData<void(enum Error, const QString &)>(20, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { QMetaType::QString, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'status'
        QtMocHelpers::PropertyData<Status>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'operations'
        QtMocHelpers::PropertyData<Operations>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'operationsReadableName'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'error'
        QtMocHelpers::PropertyData<enum Error>(22, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'errorMessage'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags, 4),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(29, 29, QMC::EnumFlags{}).add({
            {   30, State::ServiceDisconnected },
            {   31, State::ServiceConnected },
            {   32, State::ItemReady },
            {   33, State::ItemLocked },
            {   34, State::ItemNeedsSave },
            {   35, State::CollectionReady },
            {   36, State::CollectionLocked },
        }),
        // enum 'Operation'
        QtMocHelpers::EnumData<enum Operation>(37, 37, QMC::EnumFlags{}).add({
            {   38, Operation::OperationNone },
            {   39, Operation::ServiceConnecting },
            {   40, Operation::ServiceLoadingCollections },
            {   41, Operation::ItemCreating },
            {   42, Operation::ItemLoading },
            {   43, Operation::ItemLoadingSecret },
            {   44, Operation::ItemUnlocking },
            {   45, Operation::ItemSaving },
            {   46, Operation::ItemSavingLabel },
            {   47, Operation::ItemSavingSecret },
            {   48, Operation::ItemSavingAttributes },
            {   49, Operation::ItemDeleting },
            {   50, Operation::CollectionReadingDefault },
            {   51, Operation::CollectionWritingDefault },
            {   52, Operation::CollectionCreating },
            {   53, Operation::CollectionLoading },
            {   54, Operation::CollectionUnlocking },
            {   55, Operation::CollectionLocking },
            {   56, Operation::CollectionDeleting },
        }),
        // enum 'Error'
        QtMocHelpers::EnumData<enum Error>(21, 21, QMC::EnumFlags{}).add({
            {   57, Error::NoError },
            {   58, Error::ServiceConnectionError },
            {   59, Error::ServiceLoadCollectionsError },
            {   60, Error::ItemCreationError },
            {   61, Error::ItemLoadError },
            {   62, Error::ItemLoadSecretError },
            {   63, Error::ItemUnlockError },
            {   64, Error::ItemSaveError },
            {   65, Error::ItemDeleteError },
            {   66, Error::CollectionReadDefaultError },
            {   67, Error::CollectionWriteDefaultError },
            {   68, Error::CollectionCreationError },
            {   69, Error::CollectionLoadError },
            {   70, Error::CollectionUnlockError },
            {   71, Error::CollectionLockError },
            {   72, Error::CollectionDeleteError },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<StateTracker, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject StateTracker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12StateTrackerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12StateTrackerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12StateTrackerE_t>.metaTypes,
    nullptr
} };

void StateTracker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StateTracker *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->statusChanged((*reinterpret_cast<std::add_pointer_t<StateTracker::Status>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<StateTracker::Status>>(_a[2]))); break;
        case 1: _t->serviceConnectedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->operationsChanged((*reinterpret_cast<std::add_pointer_t<StateTracker::Operations>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<StateTracker::Operations>>(_a[2]))); break;
        case 3: _t->operationsReadableNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->errorChanged((*reinterpret_cast<std::add_pointer_t<enum Error>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StateTracker::*)(StateTracker::Status , StateTracker::Status )>(_a, &StateTracker::statusChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateTracker::*)(bool )>(_a, &StateTracker::serviceConnectedChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateTracker::*)(StateTracker::Operations , StateTracker::Operations )>(_a, &StateTracker::operationsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateTracker::*)(const QString & )>(_a, &StateTracker::operationsReadableNameChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateTracker::*)(Error , const QString & )>(_a, &StateTracker::errorChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<Status*>(_v) = _t->status(); break;
        case 1: *reinterpret_cast<Operations*>(_v) = _t->operations(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->operationsReadableName(); break;
        case 3: *reinterpret_cast<enum Error*>(_v) = _t->error(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->errorMessage(); break;
        default: break;
        }
    }
}

const QMetaObject *StateTracker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StateTracker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12StateTrackerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int StateTracker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void StateTracker::statusChanged(StateTracker::Status _t1, StateTracker::Status _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void StateTracker::serviceConnectedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void StateTracker::operationsChanged(StateTracker::Operations _t1, StateTracker::Operations _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void StateTracker::operationsReadableNameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void StateTracker::errorChanged(Error _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}
QT_WARNING_POP
